"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractActionHandler_1 = require("../AbstractActionHandler");
/**
 * Connects to a Mysql database using the NPM [mysql](https://github.com/mysqljs/mysql) package. This expects that
 * the database is already migrated, including an `_index_state` table. Refer to the tests for more information.
 */
class MysqlActionHandler extends AbstractActionHandler_1.AbstractActionHandler {
    constructor(updaters, effects, mysqlInstance) {
        super(updaters, effects);
        this.updaters = updaters;
        this.effects = effects;
        this.mysqlInstance = mysqlInstance;
    }
    handleWithState(handle) {
        return __awaiter(this, void 0, void 0, function* () {
            yield handle(null, { conn: this.mysqlInstance });
        });
    }
    updateIndexState(_state, block, isReplay) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mysqlInstance.query(`REPLACE INTO _index_state VALUES (0, ${block.blockNumber}, "${block.blockHash}", ${isReplay} )`);
        });
    }
    loadIndexState() {
        return __awaiter(this, void 0, void 0, function* () {
            const rows = yield this.mysqlInstance.query(`SELECT * FROM _index_state WHERE id=0`);
            if (rows.length == 0)
                return { blockNumber: 0, blockHash: "" };
            const blockNumber = rows[0].block_number;
            const blockHash = rows[0].block_hash;
            return { blockNumber, blockHash };
        });
    }
    rollbackTo(blockNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            throw Error(`Cannot roll back to ${blockNumber}; \`rollbackTo\` not implemented.`);
        });
    }
}
exports.MysqlActionHandler = MysqlActionHandler;
