"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const dockerode_1 = __importDefault(require("dockerode"));
const promise_mysql_1 = __importDefault(require("promise-mysql"));
const MysqlActionHandler_1 = require("./MysqlActionHandler");
const JsonActionReader_1 = require("../../readers/testing/JsonActionReader");
const blockchain_json_1 = __importDefault(require("./testHelpers/blockchain.json"));
const migrate = __importStar(require("./testHelpers/migrate"));
const dockerUtils = __importStar(require("./testHelpers/docker"));
const updaters_1 = __importDefault(require("./testHelpers/updaters"));
const docker = new dockerode_1.default();
const mysqlImageName = "mysql:5";
const mysqlContainerName = "mysql-action-handler-test";
const dbName = "demuxmysqltest";
const dbUser = "root";
const dbPass = "docker";
let conn;
function wait(ms) {
    return new Promise((resolve) => {
        setTimeout(resolve, ms);
    });
}
exports.wait = wait;
jest.setTimeout(30000);
beforeAll(() => __awaiter(this, void 0, void 0, function* () {
    yield dockerUtils.pullImage(docker, mysqlImageName);
    yield dockerUtils.removeMysqlContainer(docker, mysqlContainerName);
    yield dockerUtils.startMysqlContainer(docker, mysqlImageName, mysqlContainerName, dbName, dbUser, dbPass);
    conn = yield promise_mysql_1.default.createConnection({
        host: 'localhost',
        port: 3306,
        user: dbUser,
        password: dbPass,
        multipleStatements: true
    });
    yield migrate.up(conn);
}));
afterAll(() => __awaiter(this, void 0, void 0, function* () {
    yield conn.end();
    yield dockerUtils.removeMysqlContainer(docker, mysqlContainerName);
}));
describe("MysqlActionHandler", () => __awaiter(this, void 0, void 0, function* () {
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        yield migrate.reset(conn);
    }));
    it("populates database correctly", () => __awaiter(this, void 0, void 0, function* () {
        const actionReader = new JsonActionReader_1.JsonActionReader(blockchain_json_1.default);
        const actionHandler = new MysqlActionHandler_1.MysqlActionHandler(updaters_1.default, [], conn);
        const [block1, isRollback] = yield actionReader.nextBlock();
        yield actionHandler.handleBlock(block1, isRollback, actionReader.isFirstBlock);
        yield wait(500);
        let rows = yield conn.query(`SELECT * FROM todo WHERE id=1`);
        const groceries = rows[0];
        expect(groceries).toEqual({
            id: 1,
            name: "Groceries",
        });
        rows = yield conn.query(`SELECT * FROM todo WHERE id=2`);
        const placesToVisit = rows[0];
        expect(placesToVisit).toEqual({
            id: 2,
            name: "Places to Visit",
        });
        const [block2, isNotRollback] = yield actionReader.nextBlock();
        yield actionHandler.handleBlock(block2, isNotRollback, actionReader.isFirstBlock);
        yield wait(500);
        rows = yield conn.query(`SELECT * FROM task WHERE name="cookies" LIMIT 1`);
        const cookies = rows[0];
        expect(cookies).toEqual({
            id: 5,
            name: "cookies",
            completed: 0,
            todo_id: 1,
        });
        rows = yield conn.query(`SELECT * FROM task WHERE name="San Francisco" LIMIT 1`);
        const sanFrancisco = rows[0];
        expect(sanFrancisco).toEqual({
            id: 9,
            name: "San Francisco",
            completed: 0,
            todo_id: 2,
        });
        const [block3, alsoNotRollback] = yield actionReader.nextBlock();
        yield actionHandler.handleBlock(block3, alsoNotRollback, actionReader.isFirstBlock);
        yield wait(500);
        rows = yield conn.query(`SELECT * FROM task WHERE name="milk" LIMIT 1`);
        const milk = rows[0];
        rows = yield conn.query(`SELECT * FROM task WHERE name="cookies" LIMIT 1`);
        const dippedCookies = rows[0];
        expect(milk).toEqual({
            id: 4,
            name: "milk",
            completed: 1,
            todo_id: 1,
        });
        expect(dippedCookies).toEqual({
            id: 5,
            name: "cookies",
            completed: 1,
            todo_id: 1,
        });
        rows = yield conn.query(`SELECT * FROM task WHERE completed=true AND todo_id=2 LIMIT 1`);
        const hongKong = rows[0];
        expect(hongKong).toEqual({
            id: 6,
            name: "Hong Kong",
            completed: 1,
            todo_id: 2,
        });
    }));
}));
